from scrapegoat.utils import automate
from scrapegoat.utils import search_links
from scrapegoat.utils import scrape
from scrapegoat.utils import translate
import os


def getLinkData(url, topic, language='en'):
	'''
	This Function will get the data from the link and returns the similarity score and text

	Arguments: 
		url  			: str type -> url is the link to scraped 
		topic 			: str type -> topic is the topic which needs to be compared
		language		: str type -> language of the data which needs to be scraped | Range: 
									  According to google language codes

    Returns: 
    	text 			: str type -> text are the Kannada sentences scraped form web
    	mean 			: int type -> cosine similarity score 
	'''

	try:
		auto = automate()
		tex = scrape(url=url).get_text()
		sim_score = auto.get_similarityScore(tex, auto.getWiki(topic, language))
		return tex, sim_score

	except Exception as e:
		print(e)



def generateData(topic, language='en', folder_path=None, adaptive_threshold=True, threshold=0.25, n_links=100):
	'''
	This Function will generate the data from links, which is obtained via google search and 
	computes the similarity score of the scraped data and if the score is greater than the 
	threashold it will store the text in a file in the given folder

	Arguments: 
		topic 				: str type -> topic is the topic which needs to be compared
		language			: str type -> language of the data which needs to be scraped | Range: According to 
										  google language codes
		folder_path 		: str type -> folder path for storing the text scraped
		adaptive_threshold  : bool type-> True or False For Adaptive Mean Calculation | Range: [True,False]
		threshold 			: int type -> value for allowing text to be scraped  |  Range: [0,1]
		n_links				: int type -> number of links to be scraped 

	Returns:
		None
	'''

	# If Folder Path is not given then it creates a folder called data and stores the files
	# in that folder
	if folder_path==None:
		try:
			folder_path = 'data'
			os.mkdir('data')
		except:
			pass

    # If User has selected Adaptive Threashold then below code will execute 
    # For 1st 5 iterations the threashold will be 0.1 and then it changes accordingly
	if adaptive_threshold:
		# search_links Function will collect n_links if specified otherwise it takes 
		# 50 as default argument
		topic = translate(topic, language)
		links = search_links(language, topic, n_links=n_links).search_()

		for i,link in enumerate(links):
			# Looping through each links 
			text, mean = getLinkData(url=link, language=language, topic=topic)
			if mean>threshold:
			# threashold adjustment code
				if i>5:
					threshold = (threshold+mean)/2
				try:
					if text!=None:
						with open(os.path.join(folder_path,"{0000}").format(i) +".txt",'w', encoding = 'utf-8') as f:
							f.write(text)
							f.close()
				except Exception:
					pass

	# If adaptive_threshold is False then below code will execute
	else:
		# LinkCollection Function will collect n_links if specified otherwise it takes 
		# 50 as default argument 
		links = search_links(language, topic, n_links=n_links).search_()
		# links = search_links(language, topic, n_link=n_links).google()
		links = list(set(links))
		
		# Looping through each links 
		for i,link in enumerate(links):
			try:
				text, mean = getLinkData(url=link, topic=topic,language=language)
			except:
				print("Not Possible to Get the Text")

			# threashold adjustment code
			if mean>threshold:
				try:
					if text!=None:
						with open(os.path.join(folder_path,"{0000}").format(i) +".txt",'w', encoding = 'utf-8') as f:
							f.write(text)
							f.close()
				except Exception:
					pass




from bs4 import BeautifulSoup                      
import requests                                    
from googletrans import Translator                 
import wikipediaapi
from googlesearch import search
from numpy import dot
from numpy.linalg import norm
import numpy as np
from bs4.element import Comment
from sentence_transformers import SentenceTransformer, util



def translate(text, lang='en'):
    '''
    translate  it to any language using google api
    '''
    translator = Translator()
    return translator.translate(text, dest=lang).text


class automate:
    def __init__(self):
        pass

    def translate(self, text, lang='en'):
        '''
        Arguments: 
    	    text : str type -> text are the Kannada sentences scraped form web
            lang : str type -> language of the data which needs to be scraped | Range: 
									  According to google language codes
        translate  it to any language using google api
        '''
        translator = Translator()
        return translator.translate(text, dest=lang).text
    
    def get_similarityScore(self, text, topic):
        '''
        This Method caculates the similarity score of the given text and topic. It uses BERT Model 
        for getting the cosine similarity score of the sentence vectors.
        '''        
        model = SentenceTransformer('multi-qa-MiniLM-L6-cos-v1')
        query_embedding = model.encode(text)
        passage_embedding =  model.encode(topic)
        return util.dot_score(query_embedding, [passage_embedding]).detach().numpy()[0][0]

    def getWiki(self, topic, lang):
        ''''
        This method gets the wiki page text in given language
        '''
        wiki_lang = wikipediaapi.Wikipedia(lang)
        # print(self.translate(topic, lang))
        wiki_page = wiki_lang.page(self.translate(topic, lang))
        # print(wiki_page.exists())
        if wiki_page.exists():
            return wiki_page.summary.split('\n')[0]
        return topic

    def cosine_similarity(self, list_1, list_2):
        cos_sim = dot(np.asarray(list_1), np.asarray(list_2)) / (norm(np.asarray(list_1)) * norm(np.asarray(list_2)))
        return cos_sim


class search_links:
    def __init__(self, lang, topic, n_links=50, pause=2):
        self.lang = lang
        self.topic = topic
        self.links = []
        self.n_links = n_links
        self.pause = pause

    def translate(self, text, lang='en'):
        translator = Translator()
        return translator.translate(text, dest=lang).text
    
    def search_(self):
        
        # to search
        query_translated = self.translate(self.topic, self.lang)
        for link in search(query_translated,tld="co.in", num=self.n_links, stop=self.n_links,
                         pause=self.pause, lang=self.lang):
            self.links.append(link)
        
        return self.links


class scrape:
    def __init__(self, url):
        self.url = url

    def tag_visible(self, element):
        '''
        The content of the below tags will be
        added to the text
        '''
        if element.parent.name in ['h1', 'h2', 'h3', 'h4', 'p',
                                'title', 'ul', 'li', 'span']:
            return True
        if isinstance(element, Comment):
            return True
        return False

    
    def text_from_html(self, body):
        '''
        The text is extracted from the html
        '''
        soup = BeautifulSoup(body, 'html.parser')
        texts = soup.findAll(text=True)
        visible_texts = filter(self.tag_visible, texts)
        return u" ".join(t.strip() for t in visible_texts)


    def get_text(self):
        '''
        This Method returns the Language Specific text removing the english characters. We have mailnely
        used p tag 
        Arguments: 
            str type -> url
            str type -> tag='p'(default)
        Returns: 
            str type -> text (Kannada sentences scraped form web)
        '''
        url = self.url

        try:
            page = requests.get(url,  headers={'User-Agent': 'Mozilla/5.0'}).text
        except Exception:
            raise "Please check the Url path"
        text = self.text_from_html(page)
        text.replace('\n','').replace('\u200c','')
        
        return text




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="scrapegoat",
    version="1.0.0.4",
    author="Navaneeth",
    author_email="scrapegoat.python@gmail.com",
    description="Scrape data with ease",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Scrape-Goat/scrapegoat",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
   install_requires=[
      "GoogleNews>=1.5.9",
      "googletrans>=3.1.0a0",
      "scipy>=1.7.1",
      "beautifulsoup4>=4.9.3",
      "numpy>=1.21.1",
      "pandas>=1.1.5",
      "Wikipedia-API>=0.5.4",
      "sentence_transformers",
      "google"
   ],
)
